#ifndef GRWINDOW_H
#define GRWINDOW_H
#include <qapplication.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qfont.h>
#include <math.h>
#include <qpainter.h>
#define BLUE Qt::blue
#define WHITE Qt::white
#define BLACK Qt::black
#define RED Qt::red
#define GREEN Qt::green
#include <qwidget.h>
#include <vector>
#include <string>
#include "DrawArea.h"
static int Axis_Room=40;
class GrWindow: public QWidget
{
protected:
    DrawArea * _da;
    QApplication * _app;
    QGridLayout *grid;
    QPainter * _paint;
public:
    double Scale_FactorX,Scale_FactorY;
    double X_Offset, Y_Offset;
    bool _axis,_centred;
    QColor bgcolor;
    QColor fgcolor;
    double xmin,xmax,ymin,ymax;
    GrWindow( QWidget *parent=0, const char *name=0 );
    DrawArea * getDrawArea() {return  _da;};
    QApplication * getApp() {return  _app;};
    QPainter * getPainter() {return  _paint;};
    void SetApp(QApplication * app) {
        _app=app;
    	_app->setMainWidget( this );
    }	
   int  exec() {return _app->exec();};
   void SetWindow(double x1, double x2,double y1, double y2,bool axis=false,bool centred=false);
   double Xmin() { return xmin;  };
   double Xmax() { return xmax;  };
   double Ymin() { return ymin;  };
   double Ymax() { return ymax;  };
   double Xsize(){ return xmax-xmin; };
   double Ysize(){ return ymax-ymin; };
   void SetColor(QColor C) {fgcolor=C;  _paint->setPen(fgcolor); };
   void SetfgColor(QColor C) {fgcolor=C;  _paint->setPen(fgcolor);};
   void SetbgColor(QColor C) {bgcolor=C;  _paint->setBackgroundColor(bgcolor);};
   QColor GetfgColor() {return fgcolor;};
   QColor GetbgColor() {return bgcolor;};
   void plot(double x1, double y1,double x2, double y2) { 
/* Normalizzo al grafico i valori di ingresso usando il range dichiarato in
   fase di inizializzazione     */
           int gx1=int(x1*Scale_FactorX+X_Offset+Axis_Room);
           int gy1=YWINDOW-int(y1*Scale_FactorY+Y_Offset+Axis_Room);
           int gx2=int(x2*Scale_FactorX+X_Offset+Axis_Room);
           int gy2=YWINDOW-int(y2*Scale_FactorY+Y_Offset+Axis_Room);
/* collego il punto precedente con il punto attuale  */           
           _paint->drawLine(gx1,gy1,gx2,gy2);
  }
  void DrawAxis();
};

class GrInit {
protected:
   ;
   GrWindow * _window;
   
   static GrInit * _myself;
   
   GrInit(){
   
        int argc=0; char **argv=0;
        QApplication * app= new QApplication( argc, argv );
        _window=new GrWindow();
	
        _window->SetApp(app);
   
   };
public:
   static GrWindow * getWindow() {
      if (_myself==0) {
         _myself=new GrInit();
      }
      return _myself->GetWindow();
   };
   GrWindow * GetWindow() {return _window;};
   static void Close() {
      if (_myself!=0) {
//          closegraphics();
          delete _myself;
          _myself=0;
      }    
   };
   ~GrInit() {
//      closegraphics();
   };
};
class GrObj {
protected:
   QColor fgcolor;
   QColor bgcolor;
   GrWindow * _window;
   DrawArea * _area;
public:
   virtual void Draw(GrOpt opt=NONE)=0;
   GrObj() {
//      std::cout<<"create GrObj"<<std::endl;
       _window=GrInit::getWindow();
//       std::cout<<_window<<std::endl;
       _area=_window->getDrawArea();
//       std::cout<<_area<<std::endl;
//       std::cout<<this<<std::endl;
       _area->setGrObj(this);
       fgcolor=BLACK;
       bgcolor=WHITE;
       _window->show();
       
   };
   int exec() { return _window->exec(); };
   void SetColor(QColor C) {fgcolor=C; };
   void SetfgColor(QColor C) {fgcolor=C; };
   void SetbgColor(QColor C) {bgcolor=C;};
   virtual ~GrObj() {
   };
};
class Point {
public:
   double x,y;
   Point(double a, double b):x(a),y(b) {};
   Point (const Point & P):x(P.x),y(P.y) {};

};
class SimplePlot;
class Image {
public:
  typedef   std::vector<Point> Points	;
  typedef Points::const_iterator CIter   ;
  typedef Points::iterator Iter  ;
protected:
   friend class SimplePlot;
   Points plist;
   int NPoint;
   QColor Color;
public:
   Image():NPoint(0),Color(BLACK) {};
   void Add(double x,double y) {
      Point p(x,y);
      plist.push_back(p);
      NPoint++;
   };
   void setColor(QColor C) { Color=C; };
   QColor getColor() {return Color; };
   Iter begin() {return plist.begin(); }; 
   Iter end() {return plist.end(); }; 
};
class SimplePlot:public GrObj {
protected:
    std::vector<Image*> Images;
    Image * _current;
    double xmin,xmax,ymin,ymax;
    static SimplePlot * _myself;
    SimplePlot():GrObj(),_current(0){};
    SimplePlot(double x1, double x2,double y1, double y2):
              GrObj(),_current(0),xmin(x1),xmax(x2),ymin(y1),ymax(y2) {};
public:
    static SimplePlot * get() {
       if (!_myself) _myself=new SimplePlot();
       return _myself;
    
    }
    static SimplePlot * get(double x1, double x2,double y1, double y2) {
       if (!_myself) _myself=new SimplePlot(x1,x2,y1,y2);
       return _myself;
    
    }
    void SetColor(QColor C) {if (_current) _current->setColor(C);};
    void SetWindow(double x1, double x2,double y1, double y2) {
       xmin=x1;
       xmax=x2;
       ymin=y1;
       ymax=y2;
    
    };
    double Xmin() const { return xmin; };
    double Xmax() const { return xmax; };
    double Ymin() const { return ymin; };
    double Ymax() const { return ymax; };
    virtual void Draw(GrOpt opt=NONE) {
       if (opt!=SAME) {
	  _window->SetWindow(xmin,xmax,ymin,ymax,true,false);
       }
       for (std::vector<Image*>::iterator im=Images.begin(); im!=Images.end(); im++) {
          _window->SetfgColor((*im)->getColor());
//	  std::cout<<"SimplePlot "<<_window<<std::endl;
	  double oldx=0,oldy=0;
	  bool first=true;
          for (Image::Iter ip=(*im)->begin(); ip!=(*im)->end(); ip++) {
	      if (!first) _window->plot(oldx,oldy,ip->x,ip->y);
	      oldx=ip->x; oldy=ip->y;
	      first=false;
	  }


       }
    };
    void AddImage(QColor C=BLACK) {
        _current= new Image();
	_current->setColor(C);
	Images.push_back(_current);
    };
    void plot(double x, double y) { 
           
           

/* collego il punto precedente con il punto attuale  */           
           _current->Add(x,y);
	   
   }
    virtual ~SimplePlot() {exec();};
};
class Derivazione {
public:
    double (*derivata) (double x);
    static double (*func) (double x);
    static double calcola(double x);
    Derivazione (double (*f) (double x));

};
class Function1D {
protected:
   double _xmin,_xmax;
   Function1D();
public:
/* puntatori a funzioni pubblici  */
   double (*me_stessa) (double x);
   double (*primitiva) (double x);
   double (*derivata) (double x);
   double (*primitiva_inv) (double x);
/* costruttore */   
   Function1D(double (*f) (double x),double a, double b,double (*deriv) (double x)=0,
                                                        double (*integ) (double x)=0 ) {
       me_stessa=f;
       if (deriv!=0) derivata=deriv;
       else {
          Derivazione derivata_numerica(f);
          derivata=derivata_numerica.derivata;
       }   
       primitiva=integ;
       primitiva_inv=0;
       _xmin=a;
       _xmax=b;
   };
/* costruttore di copia  */
   Function1D (const Function1D & F)  {
       me_stessa=F.me_stessa;
       primitiva=F.primitiva;
       derivata=F.derivata;
       primitiva_inv=F.primitiva_inv;
       _xmin=F._xmin;
       _xmax=F._xmax;
   };   
   double operator()(double x) const {
       return me_stessa(x);
   };
/* metodi */   
   void   SetInvPrimitica(double (*intinv) (double x)) { primitiva_inv=intinv ;} ;
   double xmin()const { return _xmin  ; };
   double xmax()const { return _xmax  ; };
};
class GrFunction1D : public Function1D {
protected:
   QColor fgcolor;
   QColor bgcolor;
     double _ymin,_ymax,_step;
    GrFunction1D();
public:
    GrFunction1D(double (*f) (double x),double a, double b,double (*deriv) (double x)=0,
                                                        double (*integ) (double x)=0): 
			Function1D(f,a,b,deriv,integ) {
        _ymin=0.;
        _ymax=-1e+300;
        _step=(_xmax-_xmin)/XWINDOW;
        double x=_xmin,y,yc=0.;
        for (int k=0; k<XWINDOW; k++) {
            y=me_stessa(x);
            if (y>_ymax) _ymax=y;
            if (y<_ymin) _ymin=y;
            x+=_step;
            yc+=y;
        }
        yc/=XWINDOW;
        if (_ymax-_ymin>1.e3) {
           if (_ymax-yc>3.e2) _ymax=yc+3.e2;
           if (yc-_ymin>3.e2) _ymin=yc-3.e2;
           
        }
	
       
    };
   void SetColor(QColor C) {fgcolor=C; };
   void SetfgColor(QColor C) {fgcolor=C; };
   void SetbgColor(QColor C) {bgcolor=C;};
    virtual void Draw(GrOpt opt=NONE);
    virtual ~GrFunction1D() {};
    
};

class Istogramma {
protected:
   double step;
   double _ymin,_ymax;
   int nbin;
   int nentries;
   double underflow,overflow;
   double runsum;
   double runsum2;
   std::string Title;
   double xmin,xmax;
   double * value;
   Istogramma();
public:
   Istogramma(std::string title, int bin, double minv, double maxv):nbin(bin),Title(title),xmin(minv),xmax(maxv)  {
      if (bin<=0 || minv>maxv) {
        printf("Errore! non posso creare un istogramma con %d bin e ",bin);
        printf("minimo=%g , massimo=%g\n",minv,maxv);
      } else {
          value=new double[bin];
          for (int k=0; k<nbin; k++) value[k]=0;
          step=(xmax-xmin)/nbin;
          nentries=0;
          runsum=runsum2=0.;
          _ymin=0.;
          _ymax=-1e+300;
      }
   };
   void Fill(double x,double w=1.)  {
      int ibin=int((x-xmin)/step);
      if (ibin<0) underflow+=w;
      else if (ibin>=nbin) overflow+=w;
      else {
         value[ibin]+=w;
         nentries++;
         runsum+=w*x;
         runsum2+=w*w*x*x;
	 if (value[ibin]>_ymax) _ymax=value[ibin];
	 if (value[ibin]<_ymin) _ymin=value[ibin];
      }
   };
   double media() {
     if (nentries>0) return double(runsum/nentries);
     return 0.;
   };
   double sigma() {     
     if (nentries>0 ) {
       double arg=runsum2/nentries-media()*media();
       if (arg>=0) return sqrt(arg);
       else return 0.;
     }
     return 0.;
   };
   double Underflow() { return underflow ; };
   double Overflow() { return overflow ; };
   void Reset() {
      for (int k=0; k<nbin; k++) value[k]=0;
      nentries=0;
      runsum=runsum2=0.;
      
   };
   ~Istogramma() {
      delete[] value;
   };
   
};
class GrIstogramma : public Istogramma{
protected:
   QColor fgcolor;
   QColor bgcolor;
   GrIstogramma();
public:
    GrIstogramma(std::string title, int bin, double minv, 
                 double maxv):Istogramma(title,bin,minv,maxv) {};
    void SetColor(QColor C) {fgcolor=C; };
    void SetfgColor(QColor C) {fgcolor=C; };
    void SetbgColor(QColor C) {bgcolor=C;};
    virtual void Draw(GrOpt opt=NONE);
    virtual ~GrIstogramma() {};


};
void JobEnd();
#endif // GRWINDOW_H
